{************************************************************************}
{*                                                                      *}
{*      file       : CaseConvM.PAS                                      *}
{*                                                                      *}
{*      type       : Shell Extension                                    *}
{*                                                                      *}
{*      location   : \OwnProjects\NTFSCase                              *}
{*                                                                      *}
{*      purpose    : Convert any filename to lowercase                  *}
{*                   (Usefull for VC5 Find in Files, Unix)              *}
{*                                                                      *}
{*      author     : Lennert Ploeger (NKI / AVL)                        *}
{*                   e-mail: lennert@nki.nl                             *}
{*                   homepage: www.xs4all.nl/~lploeger                  *}
{*                                                                      *}
{*      date       : 19990328                                           *}
{*                                                                      *}
{*      portability: 32 bit delphi only (V2.0 up)                       *}
{*                                                                      *}
{*      notes      : None                                               *}
{*                                                                      *}
{************************************************************************}
{* Updates:
When            Who     What
19990328        lsp     Creation from Borland's ContextM.pas
19990329        lsp     Allowed operation on directories as well
19990424        lsp     Changed GUID
*}
unit CaseConvM;

interface

uses
  Windows, ActiveX, ComObj, ShlObj, Classes;

type
  TContextMenu = class(TComObject, IShellExtInit, IContextMenu)
  private
    DroppedFiles : TStringList;
  protected
    { IShellExtInit }
    function IShellExtInit.Initialize = SEIInitialize; // Avoid compiler warning
    function SEIInitialize(pidlFolder: PItemIDList; lpdobj: IDataObject;
      hKeyProgID: HKEY): HResult; stdcall;
    { IContextMenu }
    function QueryContextMenu(Menu: HMENU; indexMenu, idCmdFirst, idCmdLast,
      uFlags: UINT): HResult; stdcall;
    function InvokeCommand(var lpici: TCMInvokeCommandInfo): HResult; stdcall;
    function GetCommandString(idCmd, uType: UINT; pwReserved: PUINT;
      pszName: LPSTR; cchMax: UINT): HResult; stdcall;
  end;

const
  // The GUID is generated by the text editor by pressing Ctrl+Shift+G.
  // It should be unique and not used already.
  Class_ContextMenu: TGUID = '{60B859A5-FA4A-11D2-B456-7E04F2000000}';

implementation

uses ComServ, SysUtils, ShellApi, Registry;

// First function called by Explorer to query this shell extension if it
// wants to handle the selected objects
function TContextMenu.SEIInitialize(pidlFolder: PItemIDList; lpdobj: IDataObject;
  hKeyProgID: HKEY): HResult;
var
  StgMedium: TStgMedium;
  FormatEtc: TFormatEtc;
  FFileName: array[0..MAX_PATH] of Char;
  i, iFileCount: Integer;
begin
  // Fail the call if lpdobj is Nil.
  if (lpdobj = nil) then
  begin
    Result := E_INVALIDARG;
    Exit;
  end;

  with FormatEtc do
  begin
    cfFormat := CF_HDROP;
    ptd      := nil;
    dwAspect := DVASPECT_CONTENT;
    lindex   := -1;
    tymed    := TYMED_HGLOBAL;
  end;

  // Render the data referenced by the IDataObject pointer to an HGLOBAL
  // storage medium in CF_HDROP format.
  Result := lpdobj.GetData(FormatEtc, StgMedium);
  if Failed(Result) then
    Exit;
  // If only one file is selected, retrieve the file name and store it in
  // FFileName. Otherwise fail the call.
  iFileCount := (DragQueryFile(StgMedium.hGlobal, $FFFFFFFF, nil, 0));
  if (iFileCount >= 1) then
  begin
    if Assigned(DroppedFiles) then
      DroppedFiles.Clear
    else
      DroppedFiles := TStringList.Create;
    for i:=0 to iFileCount-1 do
    begin
      DragQueryFile(StgMedium.hGlobal, i, FFileName, SizeOf(FFileName));
      DroppedFiles.Add(FFileName);
    end;
    Result := NOERROR;
  end
  else
  begin
    FFileName[0] := #0;
    Result := E_FAIL;
  end;
  ReleaseStgMedium(StgMedium);
end;

// Second function called by explorer to give this shell extension the
// opportunity to extend the context menu.
function TContextMenu.QueryContextMenu(Menu: HMENU; indexMenu, idCmdFirst,
          idCmdLast, uFlags: UINT): HResult;
begin
  Result := 0; // or use MakeResult(SEVERITY_SUCCESS, FACILITY_NULL, 0);

  if ((uFlags and $0000000F) = CMF_NORMAL) or
     ((uFlags and CMF_EXPLORE) <> 0) then
  begin
    // Add one menu item to context menu
    InsertMenu(Menu, indexMenu, MF_STRING or MF_BYPOSITION, idCmdFirst,
      'LowerCase');

    // Return number of menu items added
    Result := 1; // or use MakeResult(SEVERITY_SUCCESS, FACILITY_NULL, 1)
  end;
end;

// Third function called by explorer that finally does the job: convert
// all filenames to lowercase
function TContextMenu.InvokeCommand(var lpici: TCMInvokeCommandInfo): HResult;
var
  i: Integer;
  bOk: boolean;
begin
  // Make sure we are not being called by an application
  if (HiWord(Integer(lpici.lpVerb)) <> 0) then
  begin
    Result := E_FAIL;
    Exit;
  end;

  // Make sure we aren't being passed an invalid argument number
  if (LoWord(lpici.lpVerb) <> 0) then
  begin
    Result := E_INVALIDARG;
    Exit;
  end;

  // Execute the command specified by lpici.lpVerb
  // by invoking the Delphi command line compiler.

  bOk := true;
  for i:=0 to DroppedFiles.Count-1 do
    bOk := bOk and MoveFile(PChar(DroppedFiles[i]), PChar(LowerCase(DroppedFiles[i])));
  DroppedFiles.Free;
  DroppedFiles := nil;

  if bOk then
    Result := NOERROR
  else
    Result := E_FAIL;
end;

// Function called somewhere in between. Allows the shell extension to give
// additional information that will be displayed in the status bar of
// explorer
function TContextMenu.GetCommandString(idCmd, uType: UINT; pwReserved: PUINT;
  pszName: LPSTR; cchMax: UINT): HRESULT;
begin
  if (idCmd = 0) then
  begin
    if (uType = GCS_HELPTEXT) then
      // return help string for menu item
      StrCopy(pszName, 'Convert the NTFS name to lowercase');
    Result := NOERROR;
  end
  else
    Result := E_INVALIDARG;
end;

type
  TContextMenuFactory = class(TComObjectFactory)
  public
    procedure UpdateRegistry(Register: Boolean); override;
  end;

// Registering/Unregistering semantics
procedure TContextMenuFactory.UpdateRegistry(Register: Boolean);
var
  ClassID: string;
begin
  if Register then
  begin
    inherited UpdateRegistry(Register);

    ClassID := GUIDToString(Class_ContextMenu);
    CreateRegKey('*\shellex', '', '');
    CreateRegKey('*\shellex\ContextMenuHandlers', '', '');
    CreateRegKey('*\shellex\ContextMenuHandlers\LowerCase', '', ClassID);

    CreateRegKey('Directory\shellex', '', '');
    CreateRegKey('Directory\shellex\ContextMenuHandlers', '', '');
    CreateRegKey('Directory\shellex\ContextMenuHandlers\LowerCase', '', ClassID);

    if (Win32Platform = VER_PLATFORM_WIN32_NT) then
      with TRegistry.Create do
        try
          RootKey := HKEY_LOCAL_MACHINE;
          OpenKey('SOFTWARE\Microsoft\Windows\CurrentVersion\Shell Extensions', True);
          OpenKey('Approved', True);
          WriteString(ClassID, 'Case Conversion Shell Extension');
        finally
          Free;
        end;
  end
  else
  begin
    DeleteRegKey('*\shellex\ContextMenuHandlers\LowerCase');
    DeleteRegKey('*\shellex\ContextMenuHandlers');
    DeleteRegKey('*\shellex');

    DeleteRegKey('Directory\shellex\ContextMenuHandlers\LowerCase');
    DeleteRegKey('Directory\shellex\ContextMenuHandlers');
    DeleteRegKey('Directory\shellex');

    inherited UpdateRegistry(Register);
  end;
end;

initialization
  TContextMenuFactory.Create(ComServer, TContextMenu, Class_ContextMenu,
    '', 'Case Conversion Shell Extension', ciMultiInstance,
    tmApartment);
end.
